//+------------------------------------------------------------------+
//|                                             Blau_Ergodic_MDI.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp." //  -
#property link      "http://www.mql5.com"                       //    -
#property description "Ergodic MDI-Oscillator (William Blau)"   //   mql5-
#include <WilliamBlau.mqh>                 //   (   )
//---  
#property indicator_separate_window        //     
#property indicator_buffers 6              //     
#property indicator_plots   2              //     
//---   #0 (Main)
#property indicator_label1  "Ergodic"      //     #0
#property indicator_type1   DRAW_HISTOGRAM // : DRAW_HISTOGRAM -    
#property indicator_color1  Silver         //    : Silver - 
#property indicator_style1  STYLE_SOLID    //  : STYLE_SOLID -  
#property indicator_width1  2              //  
//---   #1 (Signal Line)
#property indicator_label2  "Signal"    //     #1
#property indicator_type2   DRAW_LINE      // : DRAW_LINE - 
#property indicator_color2  Red            //    : Red - 
#property indicator_style2  STYLE_SOLID    //  : STYLE_SOLID -  
#property indicator_width2  1              //  
//---  
input int    r=20; // r -  1- EMA,   
input int    s=5;  // s -  2- EMA,    
input int    u=3;  // u -  3- EMA,     
input int    ul=3; // ul - ( )  EMA,   
input ENUM_APPLIED_PRICE AppliedPrice=PRICE_CLOSE; // AppliedPrice -  
//---     
double MainBuffer[];      // : u- 3- EMA (   #0)
double SignalBuffer[];    //  : ul- EMA   (   #1)
double PriceBuffer[];     //  
double EMA_PriceBuffer[]; // r- 1- EMA ( )
double MDBuffer[];        //  
double DEMA_MDBuffer[];   // s- 2- EMA

//---  
int    begin1, begin2, begin3, begin4, begin5; //  ,     
int    rates_total_min; //     
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---        
   //   ;     
   //   #0
   SetIndexBuffer(0,MainBuffer,INDICATOR_DATA);              // : u- 3- EMA
   //   #1
   SetIndexBuffer(1,SignalBuffer,INDICATOR_DATA);            //  : ul- EMA  
   //     ;    
   SetIndexBuffer(2,PriceBuffer,INDICATOR_CALCULATIONS);     //  
   SetIndexBuffer(3,EMA_PriceBuffer,INDICATOR_CALCULATIONS); // r- 1- EMA ( )
   SetIndexBuffer(4,MDBuffer,INDICATOR_CALCULATIONS);        //  
   SetIndexBuffer(5,DEMA_MDBuffer,INDICATOR_CALCULATIONS);   // s- 2- EMA
/*
//---   #0 (Main)
   PlotIndexSetString(0,PLOT_LABEL,"Ergodic");           //     #0
   PlotIndexSetInteger(0,PLOT_DRAW_TYPE,DRAW_HISTOGRAM); // : DRAW_HISTOGRAM -    
   PlotIndexSetInteger(0,PLOT_LINE_COLOR,Silver);        //    : Silver - 
   PlotIndexSetInteger(0,PLOT_LINE_STYLE,STYLE_SOLID);   //  : STYLE_SOLID -  
   PlotIndexSetInteger(0,PLOT_LINE_WIDTH,2);             //  
//---   #1 (Signal Line)
   PlotIndexSetString(1,PLOT_LABEL,"Signal");            //     #1
   PlotIndexSetInteger(1,PLOT_DRAW_TYPE,DRAW_LINE);      // : DRAW_LINE - 
   PlotIndexSetInteger(1,PLOT_LINE_COLOR,Red);           //    : Red - 
   PlotIndexSetInteger(1,PLOT_LINE_STYLE,STYLE_SOLID);   //  : STYLE_SOLID -  
   PlotIndexSetInteger(1,PLOT_LINE_WIDTH,1);             //  
*/
//---    
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//---
   begin1=0;           //                                -  : 
   begin2=begin1+r-1;  // or =0+(r-1)                    -  : MDBuffer[], EMA_PriceBuffer[]
   begin3=begin2+s-1;  // or =0+(r-1)+(s-1)              -  : DEMA_MDBuffer[]
   begin4=begin3+u-1;  // or =0+(r-1)+(s-1)+(u-1)        -  : MainBuffer[]
   begin5=begin4+ul-1; // or =0+(r-1)+(s-1)+(u-1)+(ul-1) -  : SignalBuffer[]
   //
   rates_total_min=begin5+1; //     
//---        #0
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,begin4);
//---        #1
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,begin5);
//---   
   string shortname=PriceName(AppliedPrice)+","+string(r)+","+string(s)+","+string(u)+","+string(ul);
   IndicatorSetString(INDICATOR_SHORTNAME,"Blau_Ergodic_MDI("+shortname+")");
//--- OnInit done
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,     //   
                const int prev_calculated, //     
                const datetime &Time[],    // Time
                const double &Open[],      // Open
                const double &High[],      // High
                const double &Low[],       // Low
                const double &Close[],     // Close
                const long &TickVolume[],  // Tick Volume
                const long &Volume[],      // Real Volume
                const int &Spread[]        // Spread
               )
  {
   int i,pos;
//---      
   if(rates_total<rates_total_min) return(0);
//---    PriceBuffer[]
   CalculatePriceBuffer(
                        AppliedPrice,        //  
                        rates_total,         //   
                        prev_calculated,     //     
                        Open,High,Low,Close, //  Open[], High[], Low[], Close[]
                        PriceBuffer          //   
                       );
//---   
   //   (pos),   /   
   //     MDBuffer[]
   if(prev_calculated==0)      //   
     {
      pos=begin2;              //  c  ,    
      for(i=0;i<pos;i++)       //   
         MDBuffer[i]=0.0;      //  
     }
   else pos=prev_calculated-1; //     
   // r- 1- EMA:   ( )   EMA_PriceBuffer[]
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin1,r,PriceBuffer,EMA_PriceBuffer);
   //     MDBuffer[]
   for(i=pos;i<rates_total;i++)
      MDBuffer[i]=PriceBuffer[i]-EMA_PriceBuffer[i];
//---   EMA
   // s- 2- EMA
   ExponentialMAOnBufferWB(
                           rates_total,     //   
                           prev_calculated, //     
                           begin2,          //         
                           s,               //  
                           MDBuffer,        //  
                           DEMA_MDBuffer    //  
                          );
   // u- 3- EMA (   #0)
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin3,u,DEMA_MDBuffer,MainBuffer);
//---  Signal Line (   #1)
   ExponentialMAOnBufferWB(rates_total,prev_calculated,begin4,ul,MainBuffer,SignalBuffer);
//--- OnCalculate done. Return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
